@@ -1,9 +1,9 @@
 /*
- * $Id: calendar.c,v 1.52 2007/02/18 20:20:10 tom Exp $
+ * $Id: calendar.c,v 1.60 2011/01/03 23:29:51 tom Exp $
  *
  *  calendar.c -- implements the calendar box
  *
- * Copyright 2001-2006,2007	Thomas E. Dickey
+ *  Copyright 2001-2010,2011	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -58,6 +58,86 @@ typedef struct _box {
     BOX_DRAW box_draw;
 } BOX;
 
+static const char *
+nameOfDayOfWeek(int n)
+{
+    static const char *table[7]
+#ifndef ENABLE_NLS
+    =
+    {
+	"Sunday",
+	"Monday",
+	"Tuesday",
+	"Wednesday",
+	"Thursday",
+	"Friday",
+	"Saturday"
+    }
+#endif
+     ;
+    const char *result = 0;
+
+    if (n >= 0 && n < 7) {
+#ifdef ENABLE_NLS
+	if (table[n] == 0) {
+	    nl_item items[7] =
+	    {
+		ABDAY_1, ABDAY_2, ABDAY_3, ABDAY_4, ABDAY_5, ABDAY_6, ABDAY_7
+	    };
+	    table[n] = nl_langinfo(items[n]);
+	}
+#endif
+	result = table[n];
+    }
+    if (result == 0) {
+	result = "?";
+    }
+    return result;
+}
+
+static const char *
+nameOfMonth(int n)
+{
+    static const char *table[12]
+#ifndef ENABLE_NLS
+    =
+    {
+	"January",
+	"February",
+	"March",
+	"April",
+	"May",
+	"June",
+	"July",
+	"August",
+	"September",
+	"October",
+	"November",
+	"December"
+    }
+#endif
+     ;
+    const char *result = 0;
+
+    if (n >= 0 && n < 12) {
+#ifdef ENABLE_NLS
+	if (table[n] == 0) {
+	    nl_item items[12] =
+	    {
+		MON_1, MON_2, MON_3, MON_4, MON_5, MON_6,
+		MON_7, MON_8, MON_9, MON_10, MON_11, MON_12
+	    };
+	    table[n] = nl_langinfo(items[n]);
+	}
+#endif
+	result = table[n];
+    }
+    if (result == 0) {
+	result = "?";
+    }
+    return result;
+}
+
 static int
 days_in_month(struct tm *current, int offset /* -1, 0, 1 */ )
 {
@@ -135,29 +215,6 @@ next_or_previous(int key, int two_d)
 static int
 draw_day(BOX * data, struct tm *current)
 {
-#ifdef ENABLE_NLS
-    char *of_week[] =
-    {
-	nl_langinfo(ABDAY_1),
-	nl_langinfo(ABDAY_2),
-	nl_langinfo(ABDAY_3),
-	nl_langinfo(ABDAY_4),
-	nl_langinfo(ABDAY_5),
-	nl_langinfo(ABDAY_6),
-	nl_langinfo(ABDAY_7)
-    };
-#else
-    static const char *const of_week[] =
-    {
-	"Sunday",
-	"Monday",
-	"Tuesday",
-	"Wednesday",
-	"Thursday",
-	"Friday",
-	"Saturday"
-    };
-#endif
     int cell_wide = MON_WIDE;
     int y, x, this_x = 0;
     int save_y = 0, save_x = 0;
@@ -179,7 +236,7 @@ draw_day(BOX * data, struct tm *current)
 		  0, (x + 1) * cell_wide, "%*.*s ",
 		  cell_wide - 1,
 		  cell_wide - 1,
-		  of_week[x]);
+		  nameOfDayOfWeek(x));
     }
 
     mday = ((6 + current->tm_mday - current->tm_wday) % 7) - 7;
@@ -218,6 +275,7 @@ draw_day(BOX * data, struct tm *current)
 	}
 	wmove(data->window, save_y, save_x);
     }
+    /* just draw arrows - scrollbar is unsuitable here */
     dlg_draw_arrows(data->parent, TRUE, TRUE,
 		    data->x + ARROWS_COL,
 		    data->y - 1,
@@ -232,39 +290,6 @@ draw_day(BOX * data, struct tm *current)
 static int
 draw_month(BOX * data, struct tm *current)
 {
-#ifdef ENABLE_NLS
-    char *months[] =
-    {
-	nl_langinfo(MON_1),
-	nl_langinfo(MON_2),
-	nl_langinfo(MON_3),
-	nl_langinfo(MON_4),
-	nl_langinfo(MON_5),
-	nl_langinfo(MON_6),
-	nl_langinfo(MON_7),
-	nl_langinfo(MON_8),
-	nl_langinfo(MON_9),
-	nl_langinfo(MON_10),
-	nl_langinfo(MON_11),
-	nl_langinfo(MON_12)
-    };
-#else
-    static const char *const months[] =
-    {
-	"January",
-	"February",
-	"March",
-	"April",
-	"May",
-	"June",
-	"July",
-	"August",
-	"September",
-	"October",
-	"November",
-	"December"
-    };
-#endif
     int month;
 
     month = current->tm_mon + 1;
@@ -276,7 +301,7 @@ draw_month(BOX * data, struct tm *current)
 		 data->height + 2, data->width + 2,
 		 menubox_border_attr, menubox_attr);	/* borders of monthbox */
     wattrset(data->window, item_attr);	/* color the month selection */
-    mvwprintw(data->window, 0, 0, "%s", months[month - 1]);
+    mvwprintw(data->window, 0, 0, "%s", nameOfMonth(month - 1));
     wmove(data->window, 0, 0);
     return 0;
 }
@@ -334,6 +359,19 @@ init_object(BOX * data,
     return 0;
 }
 
+static int
+CleanupResult(int code, WINDOW *dialog, char *prompt, DIALOG_VARS * save_vars)
+{
+    if (dialog != 0)
+	dlg_del_window(dialog);
+    dlg_mouse_free_regions();
+    if (prompt != 0)
+	free(prompt);
+    dlg_restore_vars(save_vars);
+
+    return code;
+}
+
 #define DrawObject(data) (data)->box_draw(data, &current)
 
 /*
@@ -355,13 +393,15 @@ dialog_calendar(const char *title,
 	DLG_KEYS_DATA( DLGK_FIELD_NEXT, TAB ),
 	DLG_KEYS_DATA( DLGK_FIELD_PREV, KEY_BTAB ),
 	DLG_KEYS_DATA( DLGK_GRID_DOWN,	'j' ),
+	DLG_KEYS_DATA( DLGK_GRID_DOWN,	DLGK_MOUSE(KEY_NPAGE) ),
 	DLG_KEYS_DATA( DLGK_GRID_DOWN,	KEY_DOWN ),
 	DLG_KEYS_DATA( DLGK_GRID_DOWN,	KEY_NPAGE ),
-	DLG_KEYS_DATA( DLGK_GRID_DOWN,	DLGK_MOUSE(KEY_NPAGE) ),
+	DLG_KEYS_DATA( DLGK_GRID_LEFT,	'-' ),
 	DLG_KEYS_DATA( DLGK_GRID_LEFT,  'h' ),
 	DLG_KEYS_DATA( DLGK_GRID_LEFT,  CHR_BACKSPACE ),
 	DLG_KEYS_DATA( DLGK_GRID_LEFT,  CHR_PREVIOUS ),
 	DLG_KEYS_DATA( DLGK_GRID_LEFT,  KEY_LEFT ),
+	DLG_KEYS_DATA( DLGK_GRID_RIGHT,	'+' ),
 	DLG_KEYS_DATA( DLGK_GRID_RIGHT, 'l' ),
 	DLG_KEYS_DATA( DLGK_GRID_RIGHT, CHR_NEXT ),
 	DLG_KEYS_DATA( DLGK_GRID_RIGHT, KEY_NEXT ),
@@ -392,9 +432,12 @@ dialog_calendar(const char *title,
     int state = dlg_defaultno_button();
     const char **buttons = dlg_ok_labels();
     char *prompt = dlg_strclone(subtitle);
-    int longest;
-    int mincols;
+    int mincols = MIN_WIDE;
     char buffer[MAX_LEN];
+    DIALOG_VARS save_vars;
+
+    dlg_save_vars(&save_vars);
+    dialog_vars.separate_output = TRUE;
 
     dlg_does_output();
 
@@ -429,11 +472,7 @@ dialog_calendar(const char *title,
 	    current = *localtime(&now_time);
 	}
     }
-
-    dlg_button_sizes(buttons, FALSE, &longest, &mincols);
-    mincols += (0 * MARGIN) + (dlg_button_count(buttons) * 3) - 1;
-    if (mincols < MIN_WIDE)
-	mincols = MIN_WIDE;
+    dlg_button_layout(buttons, &mincols);
 
 #ifdef KEY_RESIZE
   retry:
@@ -441,8 +480,6 @@ dialog_calendar(const char *title,
 
     dlg_auto_size(title, prompt, &height, &width, 0, mincols);
     height += MIN_HIGH - 1;
-    if (width < MIN_WIDE)
-	width = MIN_WIDE;
     dlg_print_size(height, width);
     dlg_ctl_size(height, width);
 
@@ -473,8 +510,9 @@ dialog_calendar(const char *title,
 		    DAY_HIGH + 1,
 		    draw_day,
 		    'D') < 0
-	|| DrawObject(&dy_box) < 0)
-	return DLG_EXIT_ERROR;
+	|| DrawObject(&dy_box) < 0) {
+	return CleanupResult(DLG_EXIT_ERROR, dialog, prompt, &save_vars);
+    }
 
     if (init_object(&mn_box,
 		    dialog,
@@ -484,8 +522,9 @@ dialog_calendar(const char *title,
 		    HDR_HIGH,
 		    draw_month,
 		    'M') < 0
-	|| DrawObject(&mn_box) < 0)
-	return DLG_EXIT_ERROR;
+	|| DrawObject(&mn_box) < 0) {
+	return CleanupResult(DLG_EXIT_ERROR, dialog, prompt, &save_vars);
+    }
 
     if (init_object(&yr_box,
 		    dialog,
@@ -495,8 +534,9 @@ dialog_calendar(const char *title,
 		    mn_box.height,
 		    draw_year,
 		    'Y') < 0
-	|| DrawObject(&yr_box) < 0)
-	return DLG_EXIT_ERROR;
+	|| DrawObject(&yr_box) < 0) {
+	return CleanupResult(DLG_EXIT_ERROR, dialog, prompt, &save_vars);
+    }
 
     while (result == DLG_EXIT_UNKNOWN) {
 	BOX *obj = (state == sDAY ? &dy_box
@@ -616,11 +656,23 @@ dialog_calendar(const char *title,
 	}
     }
 
-    dlg_del_window(dialog);
-    sprintf(buffer, "%02d/%02d/%0d\n",
-	    current.tm_mday, current.tm_mon + 1, current.tm_year + 1900);
+#define DefaultFormat(dst, src) \
+	sprintf(dst, "%02d/%02d/%0d", \
+		src.tm_mday, src.tm_mon + 1, src.tm_year + 1900)
+#ifdef HAVE_STRFTIME
+    if (dialog_vars.date_format != 0) {
+	size_t used = strftime(buffer,
+			       sizeof(buffer) - 1,
+			       dialog_vars.date_format,
+			       &current);
+	if (used == 0 || *buffer == '\0')
+	    DefaultFormat(buffer, current);
+    } else
+#endif
+	DefaultFormat(buffer, current);
+
     dlg_add_result(buffer);
-    dlg_mouse_free_regions();
-    free(prompt);
-    return result;
+    dlg_add_separator();
+
+    return CleanupResult(result, dialog, prompt, &save_vars);
 }
