@@ -1,9 +1,9 @@
 /*
- *  $Id: textbox.c,v 1.96 2010/01/17 18:23:56 tom Exp $
+ *  $Id: textbox.c,v 1.99 2011/01/16 22:20:34 tom Exp $
  *
- * textbox.c -- implements the text box
+ *  textbox.c -- implements the text box
  *
- * Copyright 2000-2009,2010 Thomas E.  Dickey
+ *  Copyright 2000-2010,2011	Thomas E.  Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -53,7 +53,7 @@ static long
 lseek_obj(MY_OBJ * obj, long offset, int mode)
 {
     long fpos;
-    if ((fpos = lseek(obj->fd, offset, mode)) == -1) {
+    if ((fpos = (long) lseek(obj->fd, (off_t) offset, mode)) == -1) {
 	switch (mode) {
 	case SEEK_CUR:
 	    dlg_exiterr("Cannot get file position");
@@ -72,7 +72,7 @@ lseek_obj(MY_OBJ * obj, long offset, int mode)
 static long
 ftell_obj(MY_OBJ * obj)
 {
-    return lseek_obj(obj, 0, SEEK_CUR);
+    return lseek_obj(obj, 0L, SEEK_CUR);
 }
 
 static char *
@@ -131,7 +131,7 @@ read_high(MY_OBJ * obj, size_t size_read)
 		obj->buffer_len = obj->bytes_read;
 
 		/* Allocate space for read buffer */
-		obj->buf = xalloc(obj->buffer_len + 1);
+		obj->buf = xalloc((size_t) obj->buffer_len + 1);
 	    }
 
 	} else {
@@ -200,9 +200,9 @@ tabize(MY_OBJ * obj, long val, long *first_pos)
     lseek_obj(obj, fpos - obj->fd_bytes_read, SEEK_SET);
 
     /* Allocate space for read buffer */
-    buftab = xalloc(val + 1);
+    buftab = xalloc((size_t) val + 1);
 
-    if ((read(obj->fd, buftab, val)) == -1)
+    if ((read(obj->fd, buftab, (size_t) val)) == -1)
 	dlg_exiterr("Error reading file in tabize().");
 
     begin_line = count = 0;
@@ -309,7 +309,7 @@ back_lines(MY_OBJ * obj, long n)
 		/* Really possible to move backward BUF_SIZE/2 bytes? */
 		if (fpos < BUF_SIZE / 2 + obj->fd_bytes_read) {
 		    /* No, move less than */
-		    lseek_obj(obj, 0, SEEK_SET);
+		    lseek_obj(obj, 0L, SEEK_SET);
 		    val_to_tabize = fpos - obj->fd_bytes_read;
 		} else {	/* Move backward BUF_SIZE/2 bytes */
 		    lseek_obj(obj, -(BUF_SIZE / 2 + obj->fd_bytes_read), SEEK_CUR);
@@ -340,7 +340,7 @@ back_lines(MY_OBJ * obj, long n)
 		    /* Really possible to move backward BUF_SIZE/2 bytes? */
 		    if (fpos < BUF_SIZE / 2 + obj->fd_bytes_read) {
 			/* No, move less than */
-			lseek_obj(obj, 0, SEEK_SET);
+			lseek_obj(obj, 0L, SEEK_SET);
 			val_to_tabize = fpos - obj->fd_bytes_read;
 		    } else {	/* Move backward BUF_SIZE/2 bytes */
 			lseek_obj(obj, -(BUF_SIZE / 2 + obj->fd_bytes_read), SEEK_CUR);
@@ -465,7 +465,7 @@ get_search_term(WINDOW *dialog, char *input, int height, int width)
     int fkey = 0;
     bool first = TRUE;
     int result = DLG_EXIT_UNKNOWN;
-    char *caption = _("Search");
+    const char *caption = _("Search");
     int len_caption = dlg_count_columns(caption);
     const int *indx;
     int limit;
@@ -590,7 +590,7 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 	    while ((found = match_string(obj, search_term)) == FALSE) {
 		if (obj->begin_reached)
 		    break;
-		back_lines(obj, 2);
+		back_lines(obj, 2L);
 	    }
 	}
 	if (found == FALSE) {	/* not found */
@@ -609,7 +609,7 @@ perform_search(MY_OBJ * obj, int height, int width, int key, char *search_term)
 	     */
 	    back_lines(obj, obj->page_length);
 	} else {		/* Search term found */
-	    back_lines(obj, 1);
+	    back_lines(obj, 1L);
 	}
 	/* Reprint page */
 	wattrset(obj->text, dialog_attr);
@@ -691,10 +691,10 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 
     /* Get file size. Actually, 'file_size' is the real file size - 1,
        since it's only the last byte offset from the beginning */
-    obj.file_size = lseek_obj(&obj, 0, SEEK_END);
+    obj.file_size = lseek_obj(&obj, 0L, SEEK_END);
 
     /* Restore file pointer to beginning of file after getting file size */
-    lseek_obj(&obj, 0, SEEK_SET);
+    lseek_obj(&obj, 0L, SEEK_SET);
 
     read_high(&obj, BUF_SIZE);
 
@@ -829,7 +829,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 
 		    if (fpos > obj.fd_bytes_read) {
 			/* Yes, we have to read it in */
-			lseek_obj(&obj, 0, SEEK_SET);
+			lseek_obj(&obj, 0L, SEEK_SET);
 
 			read_high(&obj, BUF_SIZE);
 		    }
@@ -849,7 +849,7 @@ dialog_textbox(const char *title, const char *file, int height, int width)
 		    read_high(&obj, BUF_SIZE);
 		}
 		obj.in_buf = obj.bytes_read;
-		back_lines(&obj, PAGE_LENGTH);
+		back_lines(&obj, (long) PAGE_LENGTH);
 		moved = TRUE;
 		break;
 	    case DLGK_GRID_UP:	/* Previous line */
